DEFINT A-Z
'*******************************
'$INCLUDE: 'ndxpack.bi'
'*******************************
'Not much to say in this area, much of the credit for
'development here goes to Mr. Ethan Winer and his
'book:
'
' "Basic Techniques & Utilities" (SEE README.TXT)
'
'Most all of the field verification and formatting is
'of my own design though.
'
'See header of module NEXUS.BAS for legend on error
'codes and such.
'
'See header of module NDXPACK.BAS and documentation
'in it's procedures, where I got alot to say.

STATIC SUB CloseDbf (FileStat%)
  'Upon closing a DBF file, the header field containing date of last
  'update must be updated, this is done here.

  Today$ = DATE$
  Ano = VAL(RIGHT$(Today$, 2))
  Dia = VAL(MID$(Today$, 4, 2))
  Mes = VAL(LEFT$(Today$, 2))
  Temp$ = CHR$(Ano) + CHR$(Mes) + CHR$(Dia)
  PUT #FileStat%, 2, Temp$
  CLOSE #FileStat%
END SUB

STATIC SUB DelRecord (FileStat%, RecNo&, ErrCode%)
 ON LOCAL ERROR GOTO DRErr
 ErrCode% = 0
 IF RecNo& <= 0 OR RecNo& > DBF.Header.TRecs THEN   'Check for valid Record No
  ErrCode% = 7: EXIT SUB                            'Set if illegal
 END IF

 Deleted$ = "*"                        'Set Deleted  flag
 MID$(DBFRecord$, 1, 1) = Deleted$     'Set flag in local Record buffer

 'Calculate record's offset
 X& = DBF.Header.FirstRec + ((RecNo& - 1) * DBF.Header.RecLen)

 'Set flag for record in DBF File
 PUT #FileStat%, X&, Deleted$

 EXIT SUB
DRErr:
 ErrCode% = 9
 EXIT SUB
 RESUME
END SUB

STATIC FUNCTION FindFile% (FileStr$)
'This is a handy little function to test for file presence by
'using DTA info and INTERRUPTX.

   FindFile% = True

   Registers.dx = VARPTR(DTAData)               'Set DTA
   Registers.ds = VARSEG(DTAData)
   Registers.ax = &H1A00
   CALL INTERRUPTX(&H21, Registers, Registers)
   
   Tmp$ = FileStr$ + CHR$(0)                    'file we seek
   Registers.ax = &H4E00
   Registers.cx = 39                            'arbtirary attribute
   Registers.dx = SADD(Tmp$)
   Registers.ds = SSEG(Tmp$)
   CALL INTERRUPTX(&H21, Registers, Registers)

   IF Registers.flags AND 1 THEN                'Check flags for error
	FindFile% = False
   ELSE
	FindFile% = True
   END IF
END FUNCTION

STATIC FUNCTION GetField% (FileStat%, FieldNum%, ErrCode%)
ON LOCAL ERROR GOTO GetRecErr2
 ErrCode% = 0

 IF FieldNum% <= 0 OR FieldNum% > DBF.Header.TFields THEN   'Chk for valid
  ErrCode% = 11: EXIT FUNCTION                              'field, exit if
 END IF                                                     'Illegal

 'Get position of first record and record length from DBF file
 'and test against current DBF.Header settings for validity

 GET #FileStat%, 9, Cnt1
 GET #FileStat%, 11, Cnt2
 IF Cnt1 <> DBF.Header.FirstRec - 1 OR Cnt2 <> DBF.Header.RecLen THEN
  ErrCode% = 10: EXIT FUNCTION
 END IF


 X = VerifyField%(FileStat%, FieldNum%)             'Get field type
 Y = DBF.Fields(FieldNum%).FOff                     'Offset in record
 Z = DBF.Fields(FieldNum%).FLen                     'and field length

 IF X = 13 OR X = 0 THEN                            'Exit on illegal
  ErrCode% = 13: EXIT FUNCTION                      'field type
 END IF

 SELECT CASE X
'******Field is Numeric
  CASE 1
   Num$ = MID$(DBFRecord$, Y, Z)
   FldVal# = CVD(MKD$(VAL(Num$)))

'******Field is a Date
  CASE 3
   FldVal$ = MID$(DBFRecord$, Y, Z)
   FldVal$ = MID$(FldVal$, 5, 2) + "-" + MID$(FldVal$, 7, 2) + "-" + MID$(FldVal$, 1, 4)
   Test& = DATEVALUE(FldVal$)

'*****Field is Text, Logical or Memo, this handles all three
  CASE ELSE
   FldVal$ = MID$(DBFRecord$, Y, Z)
   IF X = 4 AND INSTR(1, "TF", UCASE$(FldVal$)) = 0 THEN FldVal$ = "F"

 END SELECT

 GetField% = X                  'Set return value of GetField to Field type

 EXIT FUNCTION

GetRecErr2:                     'Set internal errorcode based on ERR function
 IF ERR = 52 THEN
  ErrCode% = 9
 ELSEIF ERR = 5 THEN
  ErrCode% = 12
  RESUME NEXT
 ELSE
  ErrCode% = 11
 END IF
 EXIT FUNCTION
 RESUME
END FUNCTION

STATIC FUNCTION GetFldNum% (FileStat%, FldName$, ErrCode%)
ON LOCAL ERROR GOTO GetRecErr3
 ErrCode% = 0

 FldName$ = LTRIM$(RTRIM$(UCASE$(FldName$)))         'Format FldName$


 'Get position of first record and record length from DBF file
 'and test against current DBF.Header settings for validity
 
 GET #FileStat%, 9, Cnt1
 GET #FileStat%, 11, Cnt2
 IF Cnt1 <> DBF.Header.FirstRec - 1 OR Cnt2 <> DBF.Header.RecLen THEN
  ErrCode% = 10: EXIT FUNCTION
 END IF

 'Look for Field by Name
 X = UBOUND(DBF.Fields)
 FOR I = 1 TO X
  Y = INSTR(1, DBF.Fields(I).FName, CHR$(0))
  IF UCASE$(MID$(DBF.Fields(I).FName, 1, Y - 1)) = FldName$ THEN
   GetFldNum% = I
   EXIT FOR
  END IF
 NEXT

 'If Field not found, set error code and Funct Rtn value
 IF I > X THEN GetFldNum% = 0: ErrCode% = 25

 EXIT FUNCTION

GetRecErr3:
 IF ERR = 52 THEN ErrCode% = 9 ELSE ErrCode% = 11
 EXIT FUNCTION
 RESUME
END FUNCTION

STATIC SUB GetRecord (FileStat%, RecNo&, DelState%, ErrCode%)
 ON LOCAL ERROR GOTO GetRecErr
 ErrCode% = 0

 'Get position of first record, record length and record count from DBF file
 'and test against current DBF.Header settings for validity
 
 GET #FileStat%, 5, Cnt&
 GET #FileStat%, 9, Cnt1
 GET #FileStat%, 11, Cnt2
 IF Cnt1 <> DBF.Header.FirstRec - 1 OR Cnt2 <> DBF.Header.RecLen THEN
  ErrCode% = 10: EXIT SUB
 END IF
 IF RecNo& <= 0 THEN
  ErrCode% = 7
 ELSEIF RecNo& > Cnt& THEN
  ErrCode% = 6
 ELSE
  ' Passed all tests for validity, so set buffer and address
  DBFRecord$ = SPACE$(DBF.Header.RecLen)
  Addr& = DBF.Header.FirstRec + ((RecNo& - 1) * DBF.Header.RecLen)

  GET #FileStat%, Addr&, DBFRecord$                     'Get info

  DBFRecord$ = MID$(DBFRecord$, 1, DBF.Header.RecLen)   'Size buffer to match
														'record length and

  IF MID$(DBFRecord$, 1, 1) <> "*" THEN                 'Return deleted state
   DelState% = False
  ELSE
   DelState% = True
  END IF
 END IF

 'Check to see if there was a change in record count since last access
 'this is primarily for multi-user version yet usefull as internal check

 IF DBF.Header.TRecs <> Cnt& THEN ErrCode% = 8: DBF.Header.TRecs = Cnt&

 EXIT SUB
GetRecErr:
 IF ERR = 52 THEN ErrCode% = 9 ELSE ErrCode% = 7
 EXIT SUB
 RESUME
END SUB

STATIC SUB OpenDbf (FileStr$, ErrCode%, FileStat%)
  DIM Buffer AS STRING
  ON LOCAL ERROR GOTO NotDBFile
  ErrCode% = 0

  IF FileStat% < 0 THEN                 'Chk for valid file handle,
   ErrCode% = 4: EXIT SUB
  END IF
  IF FindFile%(FileStr$) = False THEN   'and if File exists
   ErrCode% = 3: EXIT SUB
  END IF

  IF FileStat% = 0 THEN                 'chk for close on exit
   FileStat% = 254
  ELSEIF FileStat% >= 254 THEN          'if not, chk again for valid handle
   ErrCode% = 4: EXIT SUB
  END IF

  'File name comes from DTA structure during FindFile call

  DBF.FName = MID$(DTAData.FileName, 1, INSTR(1, DTAData.FileName, ".") - 1)
  
  'Open access SHARED is for network apps.

  OPEN FileStr$ FOR BINARY ACCESS READ WRITE SHARED AS FileStat%

  DBF.Status = FileStat%               'Set DBF.Status to file handle
  GET #FileStat%, 9, HLen              'and proceed to get all the info 
  DBF.Header.FirstRec = HLen + 1       'for DBF from the file header
  Buffer = SPACE$(HLen)

  GET #FileStat%, 1, Buffer
  DBF.Header.Version = ASC(Buffer)
  IF DBF.Header.Version = 131 THEN
	DBF.Header.Version = 3
	DBF.Header.Memo = -1
  ELSE
	DBF.Header.Memo = 0
  END IF

  DBF.Header.Ano = ASC(MID$(Buffer, 2, 1))
  DBF.Header.Mes = ASC(MID$(Buffer, 3, 1))
  DBF.Header.Dia = ASC(MID$(Buffer, 4, 1))

  'Spot chk for DBF file signature: Valid date

  Fecha$ = LTRIM$(STR$(DBF.Header.Mes)) + "-" + LTRIM$(STR$(DBF.Header.Dia)) + "-" + LTRIM$(STR$(DBF.Header.Ano))
  Lng& = DATEVALUE(Fecha$)


  'Proceed with getting DBF info: Fields

  DBF.Header.TRecs = CVL(MID$(Buffer, 5, 4))
  DBF.Header.RecLen = CVI(MID$(Buffer, 11, 2))
  X = (HLen - 33) / 32
  DBF.Header.TFields = X
  
  Offset = 2: BuffOff = 33: Zero$ = CHR$(0)

  'Chk for more than 100 fields

  IF DBF.Header.TFields > 100 THEN
   ErrCode% = 2: Y = 100
  ELSE
   Y = DBF.Header.TFields
  END IF

  FOR X = 1 TO Y
	FTerm = INSTR(BuffOff, Buffer, Zero$)
	DBF.Fields(X).FName = MID$(Buffer, BuffOff, FTerm - 1)
	DBF.Fields(X).FType = MID$(Buffer, BuffOff + 11, 1)
	DBF.Fields(X).FOff = Offset
	DBF.Fields(X).FLen = ASC(MID$(Buffer, BuffOff + 16, 1))
	DBF.Fields(X).Dec = ASC(MID$(Buffer, BuffOff + 17, 1))
	Offset = Offset + DBF.Fields(X).FLen
	BuffOff = BuffOff + 32
  NEXT X

  'If close on exit, then do so
  IF FileStat% = 254 THEN CLOSE #FileStat%

  EXIT SUB

NotDBFile:
 IF ERR <> 55 THEN
  CLOSE #FileStat%
  ErrCode% = 1
 ELSE
  ErrCode% = 5
 END IF
 EXIT SUB
 RESUME
END SUB

STATIC SUB PutRecord (FileStat%, RecNo&, ErrCode%)
 ON LOCAL ERROR GOTO PRErr
 ErrCode% = 0

 'Test for valid Record Number

 IF RecNo& <= 0 OR RecNo& > DBF.Header.TRecs + 1 THEN
  ErrCode% = 9
 END IF

 'Get record length & position of first rec in DBF file

 X = DBF.Header.FirstRec: Y = DBF.Header.RecLen

 'Format an output buffer from DBFRecord buffer

 Buf$ = MID$(DBFRecord$, 1, DBF.Header.RecLen)

 'Save record and update Record count if necessary

 PUT #FileStat%, ((RecNo& - 1) * Y) + X, Buf$
 IF RecNo& = DBF.Header.TRecs + 1 THEN
  DBF.Header.TRecs = RecNo&
  PUT #FileStat%, 5, RecNo&
 END IF

 EXIT SUB
PRErr:
 ErrCode% = 9
 EXIT SUB
 RESUME
END SUB

STATIC SUB UndelRec (FileStat%, RecNo&, ErrCode%)
 ON LOCAL ERROR GOTO URErr
 ErrCode% = 0

 'Chk for valid Record Number

 IF RecNo& <= 0 OR RecNo& > DBF.Header.TRecs THEN
  ErrCode% = 7: EXIT SUB
 END IF

 UnDeleted$ = " "                        'Set delete flag to blank
 MID$(DBFRecord$, 1, 1) = UnDeleted$     'Set del flag in local Record Buffer

 'Calculate address in DBF file
 X& = DBF.Header.FirstRec + ((RecNo& - 1) * DBF.Header.RecLen)

 'Set flag for record in DBF File
 PUT #FileStat%, X&, UnDeleted$

 EXIT SUB
URErr:
 ErrCode% = 9
 EXIT SUB
 RESUME
END SUB

STATIC SUB UpdateFld (FieldNum%, FldVal$, FldVal#, ErrCode%)
ON LOCAL ERROR GOTO UFErr:
 ErrCode% = 0

'Get field length and offset in record buffer

 X = DBF.Fields(FieldNum%).FLen
 Y = DBF.Fields(FieldNum%).FOff

'Based on field type, format the field appropriately

 SELECT CASE UCASE$(DBF.Fields(FieldNum%).FType)
  CASE "C", "L", "M"
   Z = LEN(FldVal$)
   IF Z < X THEN FldVal$ = FldVal$ + SPACE$(X - Z)
  CASE "D"
   Fecha& = DATEVALUE(FldVal$)
   FldVal$ = FORMAT$(Fecha&, "yyyymmdd")
  CASE "N"
   D = DBF.Fields(FieldNum%).Dec
   IF D <> 0 THEN
	Buf$ = "." + STRING$(D, "#")
	Buf$ = STRING$(X - D - 1, "#") + Buf$
   ELSE
	Buf$ = STRING$(X, "#")
   END IF
   FldVal$ = FORMAT$(FldVal#, Buf$)
   IF LEN(FldVal$) < X THEN FldVal$ = SPACE$(X - LEN(FldVal$)) + FldVal$
   IF LEN(FldVal$) > X THEN FldVal$ = MID$(FldVal$, 1, X)
 END SELECT

'Update the field in the record buffer

 MID$(DBFRecord$, Y, X) = FldVal$

 EXIT SUB
UFErr:
 ErrCode% = 12
 EXIT SUB
 RESUME
END SUB

STATIC FUNCTION VerifyField (FileStat%, FieldNum%)
 VerifyField = 0

'Get addresses for field name, class and length

 Addr1& = 33 + ((FieldNum% - 1) * 32)
 Addr2& = Addr1& + 11
 Addr3& = Addr2& + 5

'Get/set name, class and length

 Buf$ = SPACE$(10)
 Class$ = " "
 GET #FileStat%, Addr1&, Buf$
 GET #FileStat%, Addr2&, Class$
 GET #FileStat%, Addr3&, FLen

'Get true length of field name and format Class

 FTerm = INSTR(1, Buf$, CHR$(0))
 Class$ = UCASE$(Class$)

'Set function return based on Class

 SELECT CASE Class$
  CASE "N"
   VerifyField = 1
  CASE "C"
   VerifyField = 2
  CASE "D"
   VerifyField = 3
  CASE "L"
   VerifyField = 4
  CASE "M"
   VerifyField = 5
 END SELECT

 'Check for field name, class and length discrepancy with
 'values stored in DBF variable, return error through function if so.

 X = FieldNum%
 IF MID$(DBF.Fields(X).FName, 1, FTerm - 1) <> MID$(Buf$, 1, FTerm - 1) THEN VerifyField = 13
 IF UCASE$(DBF.Fields(X).FType) <> Class$ THEN VerifyField = 13
 IF DBF.Fields(X).FLen <> FLen THEN VerifyField = 13

END FUNCTION

