'****************************************************************************
'********************             HANDY.BAS              ********************
'********************            Version 1.0             ********************
'********************                by                  ********************
'********************         Raymond W. Marron          ********************
'********************      HOMONCULOUS PROGRAMMING       ********************
'********************             Mesa, AZ               ********************
'****************************************************************************

'*****************************  General Setup  ******************************

DEFINT A-Z                         'These lines must appear in your program
                                   'if you wish to cut & paste most of these
CONST TRUE = -1                    'functions into it.
CONST FALSE = NOT TRUE

DIM SHARED ecode                   'Used by the error handling routine.

'**********************  String Processing Functions  ***********************

DECLARE FUNCTION Capitalize$ (orig$)
DECLARE FUNCTION CountIn (search$, lookfor$)
DECLARE FUNCTION IsAlpha (text$)
DECLARE FUNCTION IsNum (text$)
DECLARE FUNCTION LeadZero$ (number, newlen)
DECLARE FUNCTION PadC$ (orig$, newlen)
DECLARE FUNCTION PadL$ (orig$, newlen)
DECLARE FUNCTION PadR$ (orig$, newlen)
DECLARE FUNCTION PadX$ (orig$, newlen, side$, char$)
DECLARE FUNCTION Replace$ (orig$, old$, new$)
DECLARE FUNCTION Rinstr (start, search$, lookfor$)
DECLARE FUNCTION Scramble$ (text$)
DECLARE FUNCTION Squeeze$ (orig$, char$)
DECLARE FUNCTION Strip$ (orig$, side$, char$)
DECLARE FUNCTION Stuff$ (orig$, char$, position)

'************************  File and Disk Functions  *************************

DECLARE FUNCTION CrazyName$ (ext$)
DECLARE FUNCTION Dir (filespec$, filearray$())
DECLARE FUNCTION DirExist (directory$)
DECLARE FUNCTION FileExist (filename$)
DECLARE FUNCTION FileParts$ (filespec$, operation$)
DECLARE FUNCTION HomePath$ ()
DECLARE FUNCTION MakeDir (directory$)

'************************  Screen Output Procedures  ************************

DECLARE SUB Box (row1, col1, row2, col2, boxtype$)
DECLARE SUB Center (row, text$)
DECLARE SUB Drop (text$, row, col)
DECLARE SUB Panes (row1, col1, row2, col2, row3, col3, boxtype$)
DECLARE SUB Rise (text$, row, col)
DECLARE SUB SlideL (text$, row, col, delay)
DECLARE SUB SlideR (text$, row, col, delay)
DECLARE SUB Spread (text$, row, col, delay)
DECLARE SUB TeleType (text$, delay)
DECLARE SUB Wipe (row)
DECLARE SUB WipeArea (row1, col1, row2, col2)

'************************  Simple Sorting Functions  ************************

DECLARE FUNCTION Big (num1, num2)
DECLARE SUB ASort (s$(), order)
DECLARE SUB ASortI (i(), order)

'*************************  Misc/Special Functions  *************************

DECLARE FUNCTION Evaluate$ (formula$)
DECLARE FUNCTION Dice (rolls, sides, add)

'*************************  Executable Statements  **************************

RANDOMIZE TIMER                    'This line is needed in the main module of
                                   'your code if you want to use any of the
                                   'functions that use RND().  It keeps you
                                   'from getting the same results each time
                                   'you run your program.

'*************************  Error Handling Routine  *************************

MiscErr:                           'Just make ecode contain the error code
     ecode = ERR                   'and return to the line following the one
     RESUME NEXT                   'that caused the error.  Very simple, but
                                   'also very required to use many of the
                                   'functions.

'****************************************************************************
'********************     End of Module Level Code       ********************
'****************************************************************************

SUB ASort (s$(), order)
'****************************************************************************
'Sorts an array of strings.  See SUB ASortI() for more information.
'Supporting functions: none.
'****************************************************************************

min = LBOUND(s$)                    'Find the lowest element of s$().
max = UBOUND(s$)                    'Find the highest element of s$().

SELECT CASE order
     CASE 0
          count = 0
          FOR x = min TO max
               count = count + 1
               FOR y = min TO (max - count)
                    IF s$(y + 1) < s$(y) THEN
                         SWAP s$(y + 1), s$(y)
                    END IF
               NEXT y
          NEXT x
     CASE ELSE
          count = 0
          FOR x = min TO max
               count = count + 1
               FOR y = min TO (max - count)
                    IF s$(y + 1) > s$(y) THEN
                         SWAP s$(y + 1), s$(y)
                    END IF
               NEXT y
          NEXT x
END SELECT

END SUB

SUB ASortI (i(), order)
'****************************************************************************
'Sorts an integer array.  The "order" argument determines whether the sort
'is in ascending or descending order:
'
'                   zero = ascending order (lowest to highest)
'               non-zero = descending order (highest to lowest)
'
'The array is DIRECTLY MODIFIED by this routine.
'
'This and the other sorting routines use what is called a "bubble-sort".
'It is not the most efficient way to sort, but is easy to understand.  Even
'though there are faster ways to sort, this works great on relatively small
'arrays, especially when compiled.
'
'You will notice that the FOR...NEXT loops are duplicated in the SUB.  This
'is done to increase speed.  The SELECT CASE only needs to be evaluated once,
'rather than for each iteration of the FOR...NEXT loop.  A little extra code
'never hurts when speed counts.  This is especially crucial when the program
'is being interpreted.
'
'You can make additional functions just like this one for different data
'types, such as "ASortSP()" for single precision, "ASortLI()" for long
'integers, etc.
'
'Supporting functions: none.
'****************************************************************************

min = LBOUND(i)                    'Find out the lowest element of i().
max = UBOUND(i)                    'Find out the highest element of i().

SELECT CASE order
     CASE 0
          count = 0
          FOR x = min TO max
               count = count + 1
               FOR y = min TO (max - count)
                    IF i(y + 1) < i(y) THEN
                         SWAP i(y + 1), i(y)
                    END IF
               NEXT y
          NEXT x
     CASE ELSE
          count = 0
          FOR x = min TO max
               count = count + 1
               FOR y = min TO (max - count)
                    IF i(y + 1) > i(y) THEN
                         SWAP i(y + 1), i(y)
                    END IF
               NEXT y
          NEXT x
END SELECT

END SUB

FUNCTION Big (num1, num2)
'****************************************************************************
'Returns the larger of two integers.  Not terribly handy by itself, but if
'you want to find the largest of three numbers use it like so:
'    biggest = Big(num1, Big(num2, num3) )
'
'Supporting functions: none.
'****************************************************************************

IF num1 > num2 THEN                     'If num1 = num2, it doesn't matter
     Big = num1                         'which one gets returned.
ELSE
     Big = num2
END IF

END FUNCTION

SUB Box (row1, col1, row2, col2, boxtype$)
'****************************************************************************
'Draws a box on the screen at the specified coordinates.  row1 & col1 are the
'top left corner, row2 & col2 are the bottom right corner.  The appearance of
'the box is determined by boxtype$, which can either be one character to pick
'a predefined box type or a string of six or more characters to be used as
'the actual box characters.  See the function body for the exact placement of
'the characters within the string.  It's fun to experiment with.
'
'Examples of boxtype$:  "1" or "S"  Draws a single-line box (default)
'                       "2" or "D"  Draws a double-line box
'                       "3" or "H"  Double Horizontal lines, single vertical
'                       "4" or "V"  Double Vertical lines, single horizontal
'                       "******"    Draws a box made of asterisks
'
'Supporting functions: None.
'****************************************************************************

'                     *** Define the box characters ***

IF LEN(boxtype$) >= 6 THEN                                  'User-supplied
     x$ = boxtype$                                          'box characters
ELSE
     SELECT CASE UCASE$(LEFT$(boxtype$, 1))
          CASE "2", "D"                                     'Double
               x$ = CHR$(201) + CHR$(187) + CHR$(188)
               x$ = x$ + CHR$(200) + CHR$(186) + CHR$(205)
          CASE "3", "H"                                     'Horizontal dbl.
               x$ = CHR$(213) + CHR$(184) + CHR$(190)
               x$ = x$ + CHR$(212) + CHR$(179) + CHR$(205)
          CASE "4", "V"                                     'Vertical dbl.
               x$ = CHR$(214) + CHR$(183) + CHR$(189)
               x$ = x$ + CHR$(211) + CHR$(186) + CHR$(196)
          CASE ELSE                                         'Single (default)
               x$ = CHR$(218) + CHR$(191) + CHR$(217)
               x$ = x$ + CHR$(192) + CHR$(179) + CHR$(196)
     END SELECT
END IF

'          *** Give descriptive names to the box characters ***

tlc$ = MID$(x$, 1, 1)         'Top Left Corner
trc$ = MID$(x$, 2, 1)         'Top Right Corner
brc$ = MID$(x$, 3, 1)         'Bottom Right Corner
blc$ = MID$(x$, 4, 1)         'Bottom Left Corner
ver$ = MID$(x$, 5, 1)         'Vertical line
hor$ = MID$(x$, 6, 1)         'Horizontal line

'                           *** Draw the Box ***

wide = col2 - col1 - 1

LOCATE row1, col1                                      'The top line
PRINT tlc$; STRING$(wide, hor$); trc$;
FOR x = (row1 + 1) TO (row2 - 1)                       'The sides
     LOCATE x, col1
     PRINT ver$;
     LOCATE x, col2
     PRINT ver$;
NEXT x
LOCATE row2, col1                                      'The bottom line
PRINT blc$; STRING$(wide, hor$); brc$;

END SUB

FUNCTION Capitalize$ (orig$)
'****************************************************************************
'Capitalizes the first letter of each word in a string after first converting
'the whole thing to lower case.
'Supporting functions: none.
'****************************************************************************

IF orig$ = "" THEN                      'If the original is a null string,
     Capitalize$ = ""                   'return a null string as it can not
     EXIT FUNCTION                      'be capitalized!
END IF

new$ = LCASE$(orig$)                    'Don't alter the original in case the
                                        'calling program still needs it!
                                        'And make it all lower case while
                                        'we're at it.

MID$(new$, 1, 1) = UCASE$(MID$(new$, 1, 1))  'Capitalize the first character.

FOR x = 1 TO (LEN(new$) - 1)            'Go through the string, capitalizing
                                        'anything that follows a space.
     IF MID$(new$, x, 1) = " " THEN
          MID$(new$, (x + 1), 1) = UCASE$(MID$(new$, (x + 1), 1))
     END IF

NEXT x

Capitalize$ = new$                      'Return the capitalized string.

END FUNCTION

SUB Center (row, text$)
'****************************************************************************
'Centers text on the specified row.
'Supporting functions: none.
'****************************************************************************

col = 41 - (LEN(text$) \ 2)   'Calculate the column to place text at.

IF col < 1 THEN col = 1       'Avoid an error if text$ is really long.

LOCATE row, col               'If row is illegal, its the programmer's fault!

PRINT text$;                  'Use the semicolon in case we're on row 24 -
                              'It will prevent the screen from scrolling up.
END SUB

FUNCTION CountIn (search$, lookfor$)
'****************************************************************************
'Returns the number of times that a substring is found within a string.
'Supporting functions: none.
'****************************************************************************
count = 0

IF search$ = "" OR lookfor$ = "" THEN             'Exit if searching in or
     CountIn = count                              'looking for a null string.
     EXIT FUNCTION
END IF

x = INSTR(search$, lookfor$)                      'Find the first occurrence.

DO WHILE x > 0                                    'Go through the string,
     count = count + 1                            'incrementing count and
     last = x                                     'increasing the starting
     x = INSTR((last + 1), search$, lookfor$)     'position each time the
LOOP                                              'substring is found.

CountIn = count                                   'Return the count.

END FUNCTION

FUNCTION CrazyName$ (ext$)
'****************************************************************************
'Returns a rediculous filename for use as a temporary file.  An extension may
'be provided, but defaults to ".TMP" if not specified.
'Note: There is no guarantee that this filename does not already exist, but
'if it does, whoever owns the computer probably has no idea what is in any
'of his/her files (assuming they can find them)!
'
'See the functions HomePath$() and Dir() for examples of use.
'
'Supporting functions: RANDOMIZE TIMER is needed in the main module of code
' to prevent getting the same results all the time.
'****************************************************************************

x$ = LTRIM$(RTRIM$(ext$))          'Use a temporary variable for ext$

IF x$ = "" THEN x$ = ".TMP"        'If no extension is given, default to TMP.

IF LEFT$(x$, 1) <> "." THEN        'If the extension does not begin with a
     x$ = "." + x$                 'period, add one.
END IF

x$ = LEFT$(x$, 4)                  'Trim it just in case it's too long.

avail$ = "{(`^$~!#%_@&-')}"        'Define the characters available for use.

                                   'Build the crazy filename by picking 8
crazy$ = ""                        'characters at random from avail$.
FOR x = 1 TO 8
     crazy$ = crazy$ + MID$(avail$, (INT(RND * LEN(avail$)) + 1), 1)
NEXT x

CrazyName$ = crazy$ + x$           'Return the complete filename.

END FUNCTION

FUNCTION Dice (rolls, sides, add)
'****************************************************************************
'Returns the results of the specified dice roll(s).
'If you are a role-playing gamer, you will notice that the syntax of this
'function is similar to "standard gaming notation" of dice.  For example,
'"3d6" can be easily translated to Dice(3,6,0), or "2d4+1" as Dice(2,4,1).
'
'Supporting functions: none. (RANDOMIZE TIMER)
'****************************************************************************
total = 0                               'I think you can figure this one out!
FOR x = 1 TO rolls
     total = total + INT(RND * sides) + 1 + add
NEXT x
Dice = total
END FUNCTION

FUNCTION Dir (filespec$, filearray$())
'****************************************************************************
'Returns the number of files found that match the filespec(s) given.
'filespec$ works just like the DOS "dir" command in its syntax, but unlike
'DOS, you may specify more than one filespec at a time.
'
'Examples of filespec$: "*.*"
'                       "*.TXT *.DOC"
'                       "\DOS\*.COM \UTILS\*.COM"
'
'The filearray$() array IS DIRECTLY MODIFIED by the function to contain the
'names of those files found.  This is possible because variables are passed
'by reference in QBASIC and QuickBasic (See your manual for more information
'regarding the concept of passing by reference or value).  If filearray$() is
'not large enough to hold the names of all the files that are found, the
'function will stop when filearray$() becomes full and return a negative
'number indicating the amount of the shortage.  If no files are found, Dir()
'will simply return zero.
'
'This function is especially useful with a dynamically dimensioned array, as
'shown below:
'              max = 50                 'Using a variable makes it dynamic!
'              DIM files$(1 TO max)
'              DO
'                   numfiles = Dir("*.*", files$())
'                   IF numfiles < 0 then
'                        'Uh oh! my array wasn't big enough!
'                        'Subtract a negative to get a positive:
'                        max = max - numfiles
'                        REDIM files$(1 to max)   'Resize the array to fit!
'                   END IF
'              LOOP UNTIL numfiles >= 0
'              FOR x = 1 TO numfiles
'                   PRINT files$(x)
'              NEXT x
'
'Supporting functions: CrazyName$
'****************************************************************************

fspec$ = LTRIM$(RTRIM$(filespec$))      'Trim any spaces from filespec$.

IF fspec$ = "" THEN                     'If the fspec$ is a null string,
     fspec$ = "*.*"                     'make it work like a plain "DIR".
END IF

maxspec = 10                  'Set the maximum number of filespecs that the
DIM fs$(1 TO maxspec)         'function can handle and create an array to
                              'hold them (this can easily be increased, but
                              'usually only 2 or 3 are necessary).

n = 1                              'Separate the different filespecs from
FOR x = 1 TO LEN(fspec$)           'the fspec$ string and count them.
     x$ = MID$(fspec$, x, 1)       'Each one is separated by a space.
     IF x$ = " " THEN
          n = n + 1
     ELSE
          fs$(n) = fs$(n) + x$
     END IF
     IF n > maxspec THEN           'Stop if we reach maxspec.
          n = maxspec
          EXIT FOR
     END IF
NEXT x

tempfile$ = CrazyName$("")                   'Get a temporary filename.

'For each filespec, shell out to a highly tailored DOS "DIR" command and
'redirect the command to append the temporary file.  The switches on the
'command filter out everything but the filenames themselves and sort them
'in name order.  See your DOS manual for more information regarding the
'switches and redirection.  NOTE: Tested only with MS-DOS 5.0.

FOR x = 1 TO n
     SHELL ("DIR " + fs$(x) + " /a:-d-h /o:n /b >> " + tempfile$)
NEXT x

ERASE fs$                               'Release the memory used by fs$()

maxfiles = UBOUND(filearray$)           'Find out how large filearray$() is.

fh = FREEFILE                           'Get the next available file handle.

n = 0                                   'Read the filenames from the file and
OPEN tempfile$ FOR INPUT AS #fh         'count them until the end of file is
DO WHILE NOT EOF(fh)                    'reached or filearray$ becomes full.
     LINE INPUT #fh, x$
     IF x$ <> tempfile$ THEN            'Make sure we don't capture the
          n = n + 1                     'temporary file in the array during
          filearray$(n) = x$            'a "*.*" search of the current
     END IF                             'directory!
     IF n = maxfiles AND NOT EOF(fh) THEN
          n = 0
          DO WHILE NOT EOF(fh)          'Continue counting the files if the
               LINE INPUT #fh, x$       'array gets filled to capacity and
               IF x$ <> tempfile$ THEN  'the temp file is not empty yet.
                    n = n - 1           'Watch out for that temp file again.
               END IF
          LOOP
          EXIT DO                       '(Exit the outer loop)
     END IF
LOOP
CLOSE #fh

KILL tempfile$                          'Delete the temporary file.

Dir = n                                 'Return the number of files found or
                                        'a negative # indicating the shortage
END FUNCTION

FUNCTION DirExist (directory$)
'****************************************************************************
'Returns TRUE if the directory in question exists, otherwise returns FALSE.
'Supporting functions: HomePath$ (CrazyName$)
'****************************************************************************

IF directory$ = "" THEN            'Return FALSE if directory$ is null.
     DirExist = FALSE
     EXIT FUNCTION
END IF

hp$ = HomePath$                    'Get the current directory.

d$ = directory$                    'Use a temporary variable in case we need
                                   'to alter it.

IF RIGHT$(d$, 1) = "\" THEN        'Strip trailing backslash if necessary.
     d$ = LEFT$(d$, LEN(d$) - 1)
END IF

ecode = 0                          'Reset the global error code.
ON ERROR GOTO MiscErr              'Enable the error handling routine.
CHDIR d$                           'Try to change to the path in question.
ON ERROR GOTO 0                    'Turn off the error handler.

IF ecode = 0 THEN                  'We were able to change directories.
     CHDIR hp$                     'Change back to the original directory.
     DirExist = TRUE               'Return TRUE.
ELSE
     DirExist = FALSE              'The directory does not exist.
END IF

END FUNCTION

SUB Drop (text$, row, col)
'****************************************************************************
'Prints text vertically on the screen, dropping from the specified row and
'column position.  If the length of the text would continue past row 24,
'printing will stop at that point.  See also: SUB Rise()
'
'Supporting functions: none.
'****************************************************************************

r = row                                 'We will need to alter the row value.
FOR x = 1 TO LEN(text$)
     LOCATE r, col
     PRINT MID$(text$, x, 1);
     r = r + 1
     IF r = 25 THEN EXIT FOR            'Quit if we reach the bottom of the
NEXT x                                  'screen.

END SUB

FUNCTION Evaluate$ (formula$)
'****************************************************************************
'This is a special function.  It evaluates a "formula" and returns a string
'of the value.  If an error is found within the formula (or Evaluate$ is just
'unable to handle it), Evaluate$ will return a string with a leading asterisk
'followed by a description of the error.  The best way to see what it does is
'just to experiment.  By no means am I sure that this function is completely
'bulletproof, but it will stand up to most expressions whose value doesn't
'exceed a few trillion.  This function is a good example of recursion if you
'are interested.
'
'Example:  formula$ = "10*4-(36/3)"
'          newval$ = Evaluate$(formula$)
'          IF left$(newval$,1)="*" then
'              PRINT "An error occurred!"
'              PRINT newval$                 '(Error description)
'          ELSE
'              PRINT "The value of ";formula$;" is:"; VAL(newval$)
'          END IF
'
'Supporting functions:  Squeeze$, IsNum (CountIn)
'****************************************************************************

'                     *** Preliminary Error Checking ***

f$ = Squeeze$(formula$, " ")            'Remove any spaces from the formula.

IF f$ = "" THEN                         'Evaluate a null string as zero.
     Evaluate$ = "0"
     EXIT FUNCTION
END IF

DO WHILE LEFT$(f$, 2) = "--"            'Check for leading double-minuses and
     f$ = RIGHT$(f$, LEN(f$) - 2)       'remove them (because -- = +).
LOOP

DO WHILE LEFT$(f$, 1) = "+"             'Check for leading positive signs and
     f$ = RIGHT$(f$, LEN(f$) - 1)       'remove them.
LOOP

FOR x = 1 TO LEN(f$)                              'Make sure the formula
     x$ = MID$(f$, x, 1)                          'contains only valid
     SELECT CASE ASC(x$)                          'characters by checking
          CASE 48 TO 57                           'each one's ASCII code.
               'OK - 0123456789
          CASE 45
               'OK - subtraction/negation symbol: -         'Not allowed more
               IF x$ = y$ AND z = 1 THEN evalcode = 2       'than 2 in a row
               IF x = LEN(f$) THEN evalcode = 2             'or last.
          CASE 40, 41
               'OK - left & right parentheses: ()           'Must have some-
               IF x$ = ")" AND y$ = "(" THEN evalcode = 3   'thing between!
          CASE 46
               'OK - decimal point: .                  'Not allowed to have
               IF x$ = y$ THEN evalcode = 2            'two adjacent decimals
               IF x = LEN(f$) THEN evalcode = 2        'or in last position.
          CASE 43
               'OK - plus: +                           'Not allowed adjacent
               IF y = 1 THEN evalcode = 2              'to another operator
               IF x = LEN(f$) THEN evalcode = 2        'or in last position.
          CASE 37, 42, 43, 47, 92, 94
               'OK - operators: % * / \ ^              'Not allowed in first
               IF x = 1 THEN evalcode = 2              'position, adjacent to
               IF y > 0 THEN evalcode = 2              'another operator, or
               IF x = LEN(f$) THEN evalcode = 2        'in last position.
          CASE ELSE
               'NOT OK - is some other character!
               evalcode = 1
     END SELECT
     IF evalcode > 0 THEN GOTO EvalErrorExit
     z$ = y$                                      'Record the two previous
     z = y                                        'characters and whether
     y$ = x$                                      'they were an operator or
     SELECT CASE ASC(y$)                          'a left parentheses.
          CASE 40      'Left parentheses
               y = 2
          CASE 45, 37, 42, 43, 47, 92, 94
               y = 1   'An operator
          CASE ELSE
               y = 0   'Something else
     END SELECT
NEXT x

y$ = "": y = 0: z = 0
FOR x = 1 TO LEN(f$)                                   'Check for mismatched
     x$ = MID$(f$, x, 1)                               ' parentheses: unequal
     IF x$ = "(" THEN                                  ' numbers of each or
          y = y + 1                                    ' ending with a left
          y$ = x$                                      ' parentheses.
     ELSEIF x$ = ")" THEN
          z = z + 1
          y$ = x$
     END IF
NEXT x
IF y <> z OR y$ = "(" THEN evalcode = 3: GOTO EvalErrorExit

'                 *** Evaluate between parentheses first ***

DO
     start = 0
     FOR x = 1 TO LEN(f$)
          x$ = MID$(f$, x, 1)
          IF x$ = "(" THEN                        'Find a complete pair.
               start = x
          ELSEIF x$ = ")" THEN
               IF start = 0 THEN                  'Not allowed to have a )
                    evalcode = 3                  'without a ( !
                    GOTO EvalErrorExit
               END IF
               y = x - start - 1                  'Extract the expression
               mf$ = MID$(f$, start + 1, y)       'between the parentheses
               lf$ = LEFT$(f$, start - 1)         'and recurse the function
               rf$ = RIGHT$(f$, LEN(f$) - x)      'to get its value.  Then
               mf$ = Evaluate$(mf$)               '(assuming no errors) put
               IF LEFT$(mf$, 1) = "*" THEN        'the formula back together,
                    Evaluate$ = mf$               'replacing the parentheses
                    EXIT FUNCTION                 'with the value of the
               END IF                             'expression.
               f$ = lf$ + mf$ + rf$
               EXIT FOR                           'Start at the beginning
          END IF                                  ' of the formula again.
     NEXT x
LOOP UNTIL start = 0               'Loop until no more parentheses are found.

'                  *** Evaluate the rest of the formula ***

FOR pass = 1 TO 4                       'Make four passes through the
     SELECT CASE pass                   ' formula, performing calculations
          CASE 1                        ' in order of operator precedence.
               op1$ = "^"
               op2$ = "^"               'Exponentiation only first
          CASE 2
               op1$ = "*"               'Multiplication & Division second
               op2$ = "/"
          CASE 3
               op1$ = "\"               'Integer and Modulus Division third
               op2$ = "%"
          CASE 4
               op1$ = "+"               'Addition and Subtraction last
               op2$ = "-"
     END SELECT
     DO
          op = 0
          FOR x = 1 TO LEN(f$)               'Search for desired operators.
               x$ = MID$(f$, x, 1)
               IF x$ = op1$ OR x$ = op2$ AND x > 1 THEN     'Beware of the
                    op = x                                  ' leading minus!
                    GOSUB EvalCalcs          'Found one!  Do the math and
                    EXIT FOR                 'start from the beginning again.
               END IF
          NEXT x
     LOOP UNTIL op = 0                  'Go through the formula until none of
NEXT pass                               ' the specified operators are found.

Evaluate$ = f$                          'Return the boiled down formula.
EXIT FUNCTION

'   *** The following section is where the values on either side of ***
'   ***   the operator are parsed out and the actual math occurs.   ***

EvalCalcs:

     operator$ = MID$(f$, op, 1)        'Pull the operator.

     v1$ = "": lf = 1                   'Pull the first value:
     FOR y = (op - 1) TO 1 STEP -1      'Look to the left of the operator
          y$ = MID$(f$, y, 1)           ' one char at a time until the next
          IF IsNum(y$) THEN             ' operator (or beginning) is found.
               v1$ = y$ + v1$           'Add the numeric character to the
               lf = y                   ' first value and record position.
          ELSEIF y$ <> "-" THEN
               EXIT FOR                 'Found a non-minus operator - stop.
          ELSEIF y$ = "-" THEN
               IF y = 1 THEN
                    v1$ = y$ + v1$      'Leading minus in first position.
                    lf = 1              'Add it, record position and stop.
                    EXIT FOR
               ELSEIF IsNum(MID$(f$, y - 1, 1)) THEN
                    EXIT FOR            'Next char is a number - stop.
               END IF                   '(We were checking for double negs.)
          END IF
     NEXT y
                                        'Pull the second value:
     v2$ = MID$(f$, op + 1, 1)          'Take the very next character in case
     rf = op + 1                        ' it is a leading minus sign
     FOR y = (op + 2) TO LEN(f$)        'Look to the right of the operator
          y$ = MID$(f$, y, 1)           ' one char at a time until the next
          IF IsNum(y$) THEN             ' operator (or the end) is found.
               v2$ = v2$ + y$           'Add the numeric character to the
               rf = y                   'second value and record position.
          ELSE
               EXIT FOR                 'Next operator found - stop looking.
          END IF
     NEXT y
   
     ecode = 0                          'Prepare to trap any math errors.
     ON ERROR GOTO MiscErr
     v1# = VAL(v1$)                     'Convert the strings into double-
     v2# = VAL(v2$)                     ' precision values.
     SELECT CASE operator$              'Perform the actual math depending on
          CASE "+"                      ' the operator.
               v# = v1# + v2#
          CASE "-"
               v# = v1# - v2#
          CASE "*"
               v# = v1# * v2#
          CASE "/"
               v# = v1# / v2#
          CASE "\"
               v# = v1# \ v2#
          CASE "%"
               v# = v1# MOD v2#
          CASE "^"
               v# = v1# ^ v2#
     END SELECT
     ON ERROR GOTO 0                    'Disable error trapping.
     IF ecode > 0 THEN                  'Exit if any errors occurred.
          evalcode = ecode
          GOTO EvalErrorExit
     END IF
   
     mf$ = STR$(v#)                     'Turn the result back into a string
     mf$ = Squeeze$(mf$, " ")           ' and remove any spaces.

     IF INSTR(mf$, "D") > 0 THEN        'Make sure value has not been
          evalcode = 6                  ' converted into scientific notation
          GOTO EvalErrorExit            ' by QuickBasic's math routines
     END IF                             ' becuase VAL() cant handle it (and I
                                        ' don't care to deal with it just
                                        ' yet!).
    
     lf$ = LEFT$(f$, lf - 1)            'Pull the strings from around the
     rf$ = RIGHT$(f$, LEN(f$) - rf)     ' calculation and put them back
     f$ = lf$ + mf$ + rf$               ' together, replacing the calculation
                                        ' with its value.
     RETURN

'     *** In case of an error, the error code is translated into a ***
'     *** meaningful phrase and the function returns the message.  ***

EvalErrorExit:
   
     x$ = "* ERROR" + STR$(evalcode) + " * "
     SELECT CASE evalcode
          CASE 1
               x$ = x$ + "Invalid character in position" + STR$(x) + ": " + f$
          CASE 2
               x$ = x$ + "Invalid placement of operator in position" + STR$(x) + ": " + f$
          CASE 3
               x$ = x$ + "Formula contains mismatched parentheses."
          CASE 6
               x$ = x$ + "Overflow - values too large or too small"
          CASE 11
               x$ = x$ + "Division by zero"
          CASE ELSE
               x$ = x$ + "Unexpected error"
     END SELECT
     Evaluate$ = x$                'Return a string beginning with * so the
     EXIT FUNCTION                 ' user can easily determine if an error
                                   ' occurred. i.e.: IF LEFT$(r$,1)="*"...
END FUNCTION

FUNCTION FileExist (filename$)
'****************************************************************************
'Returns TRUE if the file in question exists, FALSE if it doesn't.
'The filename$ argument may be an extended filename, including the drive and
'path.  If no path is specified, only the current directory is searched.
'
'Supporting functions: FileParts$ (Rinstr), CrazyName$
'****************************************************************************

IF filename$ = "" THEN             'Return FALSE if a null string is passed.
     FileExist = FALSE
     EXIT FUNCTION
END IF

p$ = FileParts$(filename$, "P")    'Extract the path from filename$ (if any).

tempname$ = p$ + CrazyName$("")    'Get a temporary filename that will reside
                                   'in the same directory.

ecode = 0                          'Reset the global error code.
ON ERROR GOTO MiscErr              'Enable the error handlng routine.

NAME filename$ AS tempname$        'Try to rename the new filename.

ON ERROR GOTO 0                    'Turn off the error handler.

SELECT CASE ecode                  'The error code will give us the results.
     CASE 0
          'No error - the file was renamed, therefore it exists.
          'Rename it back and return TRUE.
          NAME tempname$ AS filename$
          FileExist = TRUE
     CASE ELSE
          'The rename was unsuccessful - the file does not exist.
          FileExist = FALSE
END SELECT

END FUNCTION

FUNCTION FileParts$ (filespec$, operation$)
'****************************************************************************
'Returns a specified part of an extended filename or other filespec type
'string.  This could have been grouped with the string processing functions,
'but was not because it deals strictly with file type stuff.
'The return value depends upon the value of the operation$ argument:
'
'Example: filespec$ = "C:\GAMES\SAVEGAME.001"
'         FileParts$(filespec$,"P")  -->  "C:\GAMES\"       (Path)
'         FileParts$(filespec$,"F")  -->  "SAVEGAME.001"    (Filename)
'         FileParts$(filespec$,"E")  -->  "001"             (Extension)
'         FileParts$(filespec$,"N")  -->  "SAVEGAME"        (fileName)
'         FileParts$(filespec$,"D")  -->  "C:"              (Drive)
'
'         filespec$ = "HOMEWORK.TXT"
'         FileParts$(filespec$,"P")  -->  ""   Returns null if the requested
'         FileParts$(filespec$,"D")  -->  ""   info is not part of filespec$
'
'Note that paths are returned with a trailing backslash.
'
'Limitations:  FileParts$() assumes that all filenames have an extension.
' If you pass the function a filename that does not contain a period (.),
' the function will assume it is a directory.
' Example: "C:\GAMES\README" will be interpreted as "C:\GAMES\README\"
'
'Supporting functions: Rinstr
'****************************************************************************

fspec$ = LTRIM$(RTRIM$(filespec$)) 'Trim any spaces off filespec$

IF fspec$ = "" THEN                'If the filespec is a null string, return
     FileParts$ = ""               'a null string.
     EXIT FUNCTION
END IF

part$ = ""                         'Initial calculations
dot = INSTR(filespec$, ".")
IF dot = 0 THEN                         'If there's no dot, it must be a
     fspec$ = fspec$ + "\"              'pathname only - add a trailing
END IF                                  'backslash.
lastslash = Rinstr(0, filespec$, "\")
colon = INSTR(filespec$, ":")

SELECT CASE UCASE$(LEFT$(operation$, 1))
     CASE "F"                                                    'Filename
          IF lastslash > 0 THEN         'Find the left boundary  '(w/ ext.)
               l = lastslash + 1
          ELSEIF colon > 0 THEN
               l = colon + 1
          ELSE                '(Assume the whole thing is a filename)
               l = 1
          END IF
          part$ = MID$(filespec$, l)
     CASE "N"                                                    'fileName
          IF lastslash > 0 THEN         'Find the left boundary  '(w/o ext.)
               l = lastslash + 1
          ELSEIF colon > 0 THEN
               l = colon + 1
          ELSE
               l = 1
          END IF
          r = dot - 1                   'Find the right boundary
          part$ = MID$(filespec$, l, (r - l + 1))
     CASE "E"                                                    'Extension
          IF dot > 0 THEN
               part$ = MID$(filespec$, (dot + 1))
          END IF
     CASE "P"                                                    'Path
          IF lastslash > 0 THEN                   'Regular pathname?
               part$ = LEFT$(filespec$, lastslash)
          ELSEIF colon > 0 THEN                   'Drive letter only?
               part$ = LEFT$(filespec$, colon) + "\"
          END IF
     CASE "D"                                                    'Drive
          IF colon > 0 THEN
               part$ = LEFT$(filespec$, colon)
          END IF
     CASE ELSE                                                   'What???
          part$ = ""
END SELECT

FileParts$ = part$            'Return the requested information.

END FUNCTION

FUNCTION HomePath$
'****************************************************************************
'Returns the name of the current DOS directory.
'Supporting functions: CrazyName$
'****************************************************************************

tempfile$ = CrazyName$("")         'Get a temporary filename.

SHELL ("CD > " + tempfile$)        'Shell out and redirect the output of the
                                   'DOS command "CD" to the temporary file.

fh = FREEFILE                      'Get the next available file handle.

OPEN tempfile$ FOR INPUT AS #fh    'Open the temporary file.
LINE INPUT #fh, hp$                'Read in the one and only line.
CLOSE #fh                          'Close the file.

KILL tempfile$                     'Delete the temporary file.

HomePath$ = hp$                    'Return the name of the current directory.

END FUNCTION

FUNCTION IsAlpha (text$)
'****************************************************************************
'Returns TRUE if the text contains only letters and spaces, otherwise FALSE.
'Supporting functions: none.
'****************************************************************************

IF text$ = "" THEN                           'If text$ is a null string,
     IsAlpha = TRUE                          'return TRUE (What the heck,
     EXIT FUNCTION                           'it's not numeric!)
END IF

allowed$ = " ABCDEFGHIJKLMNOPQRSTUVWXYZ"     'Define the allowed characters.
                                             '(The space is there on purpose)
FOR x = 1 TO LEN(text$)
                                             'Go through text$ one character
     c$ = UCASE$(MID$(text$, x, 1))          'at a time (upper case).

     IF INSTR(allowed$, c$) = 0 THEN         'If any character does not
          IsAlpha = FALSE                    'appear within allowed$, it is
          EXIT FUNCTION                      'not alpha.
     END IF

NEXT x

IsAlpha = TRUE                               'All tests passed to get here.

END FUNCTION

FUNCTION IsNum (text$)
'****************************************************************************
'Returns TRUE if the text contains only numeric data, FALSE otherwise.
'
'Examples:  IsNum("12")   --> TRUE      IsNum("ABC") --> FALSE
'           IsNum("-6")   --> TRUE      IsNum(" ")   --> FALSE
'           IsNum("1.3")  --> TRUE      IsNum("-")   --> FALSE
'                                       IsNum(".")   --> TRUE
'Supporting functions: CountIn
'****************************************************************************

IsNum = FALSE                      'Assume the worst

t$ = LTRIM$(RTRIM$(text$))         'Trim any spaces from text$.

IF t$ = "" THEN EXIT FUNCTION      'Return FALSE if there's nothing left.

IF t$ = "-" THEN EXIT FUNCTION     'A minus by itself is an operator, not
                                   'a number (But a decimal is, as far as
                                   'the function Evaluate$() is concerned -
                                   'if you use Evaluate$(), DON'T ALTER THIS!)

'Check each character's ASCII code to make sure it is one of the following:
'A minus sign (45), a decimal (46), or a digit (48-57).  (ASCII 47 is /)

FOR x = 1 TO LEN(t$)
     a = ASC(MID$(t$, x, 1))
     IF (a < 45) OR (a > 57) OR (a = 47) THEN EXIT FUNCTION
NEXT x

'Make sure that there is not more than one minus sign and if there is one,
'that it is only in the first position.  Also make sure there is only one
'decimal point.

IF INSTR(t$, "-") > 1 THEN EXIT FUNCTION
IF CountIn(t$, "-") > 1 THEN EXIT FUNCTION
IF CountIn(t$, ".") > 1 THEN EXIT FUNCTION

IsNum = TRUE                       'All tests must pass to reach this point.

END FUNCTION

FUNCTION LeadZero$ (number, newlen)
'****************************************************************************
'"Stringifys" an integer and pads it on the left with leading zeros up to the
'desired length.  This function was created mainly due to PRINT USING's
'inability to add leading zeros (But you can add asterisks!  Gee, I use that
'a lot.  NOT!!).
'
'Supporting functions: none.
'****************************************************************************

new$ = LTRIM$(STR$(number))        'Turn the number into a string.

l = LEN(new$)                      'Find the length of the new string.

IF newlen <= l THEN                'If the string equal to or longer than the
     LeadZero$ = new$              'new length, return the unmodified string.
     EXIT FUNCTION
END IF

IF number < 0 THEN                 'If the number is negative, remove the -.
     new$ = RIGHT$(new$, (l - 1))
END IF

diff = newlen - l                  'Find the difference in lengths.

new$ = STRING$(diff, "0") + new$   'Pad the string with leading zeros.

IF number < 0 THEN                 'If the number is negative, replace the -.
     new$ = "-" + new$             'Its length will now be equal to newlen
END IF                             'since we measured its length before
                                   'removing the -.

LeadZero$ = new$                   'Return the finished product.

END FUNCTION

FUNCTION MakeDir (directory$)
'****************************************************************************
'Creates a DOS directory.  The directory may be more than one level deep, and
'can be on a different drive.
'Returns TRUE if successful, FALSE if unable to create the directory.
'
'Examples:  MakeDir("GAMES")  Creates a subdirectory "GAMES" off the current
'                             directory.
'           MakeDir("C:\SCIENCE\DATA")  Creates the specified directory.
'
'You will notice that part of this function nearly duplicates the function
'DirExist().  This is because making repeated calls to DirExist causes a lot
'of disk activity and would make this function very slow.
'
'Supporting functions:  HomePath$ (CrazyName$)
'****************************************************************************

d$ = directory$                    'Use a temporary variable in case we need
                                   'to alter it.

IF d$ = "" THEN                    'If a null string is passed, return FALSE.
     MakeDir = FALSE
     EXIT FUNCTION
END IF

hp$ = HomePath$                    'Find out our current directory.

IF RIGHT$(d$, 1) = "\" THEN        'Strip the trailing backslash if there is
     d$ = LEFT$(d$, LEN(d$) - 1)   'one.
END IF

newdir$ = d$                       'See if the directory already exists.
GOSUB MDCheckExist                 'If it does, return TRUE and take credit
IF there THEN                      'for creating it!
     MakeDir = TRUE
     EXIT FUNCTION
END IF

MakeDir = FALSE                    'Assume the worst until we're done.

'This next part is kind of tricky:  Go through the new directory one
'character at a time, adding each one to the variable newdir$.  Each time a
'backslash is encountered (unless it represents the root directory: x>3), see
'if newdir$ exists.  If it doesn't, try to create it and exit if there is an
'error.  The final "MKDIR d$" is there because d$ does not end with a
'backslash and would escape detection in the FOR...NEXT loop.

newdir$ = ""
FOR x = 1 TO LEN(d$)
     c$ = MID$(d$, x, 1)
     IF c$ = "\" AND x > 3 THEN
          GOSUB MDCheckExist
          IF NOT there THEN
               ecode = 0
               ON ERROR GOTO MiscErr
               MKDIR newdir$
               ON ERROR GOTO 0
               IF ecode > 0 THEN EXIT FUNCTION
          END IF
     END IF
     newdir$ = newdir$ + c$
NEXT x

ecode = 0
ON ERROR GOTO MiscErr
MKDIR d$
ON ERROR GOTO 0
IF ecode > 0 THEN EXIT FUNCTION

MakeDir = TRUE                     'Everything went OK to get to this point.

EXIT FUNCTION                      'Don't fall through into the GOSUB area!
                                   
'*** The following section is a custom DirExist() just for this function: ***

MDCheckExist:

     there = FALSE                 'Assume the worst.
    
     ecode = 0                     'Reset the global error code.
     ON ERROR GOTO MiscErr         'Enable the error handling routine.
     CHDIR newdir$                 'Try to change to the path in question.
     ON ERROR GOTO 0               'Turn off the error handler.

     IF ecode = 0 THEN             'We were able to change directories.
          CHDIR hp$                'Change back to the original directory
          there = TRUE             'and return TRUE.
     END IF

     RETURN

END FUNCTION

FUNCTION PadC$ (orig$, newlen)
'****************************************************************************
'Pads a string with spaces equally on each side up to the new length.
'(It's name means "PadCenter")
'
'Example:  PadC$("Hello!",10) -->  "  Hello!  "
'Supporting functions: none.
'****************************************************************************

l = LEN(orig$)                     'Find the length of the original string.

IF newlen <= l THEN                'If the original is longer than or equal
     PadC$ = orig$                 'to the new length, return the original.
     EXIT FUNCTION
END IF

IF orig$ = "" THEN                 'If the original is a null string, return
     PadC$ = SPACE$(newlen)        'a string of spaces.
     EXIT FUNCTION
END IF

diff = newlen - l                  'Find the difference in lengths.

ls = diff \ 2                      'Calculate the amount of spaces to be
rs = diff - ls                     'added to the left and right sides.

PadC$ = SPACE$(ls) + orig$ + SPACE$(rs)      'Return the padded string.

END FUNCTION

FUNCTION PadL$ (orig$, newlen)
'****************************************************************************
'Pads a string with spaces on the left up to the new length.
'Example: PadL$("Hello!",10) --> "    Hello!"
'Supporting functions: none.
'****************************************************************************

l = LEN(orig$)                     'Find the length of the original string.

IF newlen <= l THEN                'If the original is longer than or equal
     PadL$ = orig$                 'to the new length, return the original.
     EXIT FUNCTION
END IF

IF orig$ = "" THEN                 'If the original is a null string, return
     PadL$ = SPACE$(newlen)        'a string of spaces.
     EXIT FUNCTION
END IF

diff = newlen - l                  'Find the difference in lengths.

PadL$ = SPACE$(diff) + orig$       'Return the padded string.

END FUNCTION

FUNCTION PadR$ (orig$, newlen)
'****************************************************************************
'Pads a string with spaces on the right up to the new length.
'Example: PadR$("Hello!",10) --> "Hello!    "
'Supporting functions: none.
'****************************************************************************

l = LEN(orig$)                     'Find the length of the original string.

IF newlen <= l THEN                'If the original is longer than or equal
     PadR$ = orig$                 'to the new length, return the original.
     EXIT FUNCTION
END IF

IF orig$ = "" THEN                 'If the original is a null string, return
     PadR$ = SPACE$(newlen)        'a string of spaces.
     EXIT FUNCTION
END IF

diff = newlen - l                  'Find the difference in lengths.

PadR$ = orig$ + SPACE$(diff)       'Return the padded string.

END FUNCTION

FUNCTION PadX$ (orig$, newlen, side$, char$)
'****************************************************************************
'Pads a string with a specified character on the specified side(s) up to the
'new length.  A more flexible version of the other "Pad" functions (all of
'which can be duplicated by this function).
'
'Examples: PadX$("Hello!",10,"R"," ") --> "Hello!    "
'          PadX$("$500",10,"L","*")   --> "******$500"
'          PadX$("WOW",20,"C","!")    --> "!!!!!!!!WOW!!!!!!!!!"
'
'Supporting functions: none.
'****************************************************************************

l = LEN(orig$)                     'Find the length of the original string.

IF newlen <= l THEN                'If the original is longer than or equal
     PadX$ = orig$                 'to the new length, return the original.
     EXIT FUNCTION
END IF

IF orig$ = "" THEN                      'If the original is a null string,
     PadX$ = STRING$(newlen, char$)     'return a string of char$.
     EXIT FUNCTION
END IF

diff = newlen - l                  'Find the difference in lengths.

SELECT CASE UCASE$(LEFT$(side$, 1))     'Pad the string on the chosen side.
     CASE "L"                                          'Pad Left
          PadX$ = STRING$(diff, char$) + orig$
     CASE "R"                                          'Pad Right
          PadX$ = orig$ + STRING$(diff, char$)
     CASE ELSE                                         'Pad Center (default)
          ls = diff \ 2
          rs = diff - ls
          PadX$ = STRING$(ls, char$) + orig$ + STRING$(rs, char$)
END SELECT

END FUNCTION

SUB Panes (row1, col1, row2, col2, row3, col3, boxtype$)
'****************************************************************************
'Draws a box on the screen at the specified coordinates.  row1 & col1 are the
'top left corner, row2 & col2 are the bottom right corner.  row3 & col3 are
'parameters specifying where the box is to be split horizontally and/or
'vertically.  If either or both row3 or col3 are zero, the box will not be
'split in that direction (and would then duplicate SUB Box() ).  Experiment
'with it.
'
'The appearance of the box is determined by boxtype$, which can either be one
'character to pick a predefined box type or a string of 11 or more characters
'to be used as the actual box characters.  See the function body for the
'exact placement of the characters within the string.
'
'Note: The first 6 characters in the boxtype$ string have the same placement
'as those in SUB Box(), so a boxtype$ string of 11 characters can be used
'interchangeably between the two procedures.
'
'Examples of boxtype$: "1" or "S"    Draws a single-line box (default)
'                      "2" or "D"    Draws a double-line box
'                      "3" or "H"    Double horizontal lines, single vertical
'                      "4" or "V"    Double vertical lines, single horizontal
'                      "***********" Draws a box made of asterisks
'
'Supporting functions: none.
'****************************************************************************

'                     *** Define the box characters ***

IF LEN(boxtype$) >= 11 THEN                                 'User-supplied
     x$ = boxtype$                                          'box characters
ELSE
     SELECT CASE UCASE$(LEFT$(boxtype$, 1))
          CASE "2", "D"                                     'Double
               x$ = CHR$(201) + CHR$(187) + CHR$(188)
               x$ = x$ + CHR$(200) + CHR$(186) + CHR$(205)
               x$ = x$ + CHR$(203) + CHR$(185) + CHR$(202)
               x$ = x$ + CHR$(204) + CHR$(206)
          CASE "3", "H"                                     'Horizontal dbl.
               x$ = CHR$(213) + CHR$(184) + CHR$(190)
               x$ = x$ + CHR$(212) + CHR$(179) + CHR$(205)
               x$ = x$ + CHR$(209) + CHR$(181) + CHR$(207)
               x$ = x$ + CHR$(198) + CHR$(216)
          CASE "4", "V"                                     'Vertical dbl.
               x$ = CHR$(214) + CHR$(183) + CHR$(189)
               x$ = x$ + CHR$(211) + CHR$(186) + CHR$(196)
               x$ = x$ + CHR$(210) + CHR$(182) + CHR$(208)
               x$ = x$ + CHR$(199) + CHR$(215)
          CASE ELSE                                         'Single (default)
               x$ = CHR$(218) + CHR$(191) + CHR$(217)
               x$ = x$ + CHR$(192) + CHR$(179) + CHR$(196)
               x$ = x$ + CHR$(194) + CHR$(180) + CHR$(193)
               x$ = x$ + CHR$(195) + CHR$(197)
     END SELECT
END IF

'           *** Give descriptive names to the box characters ***

tlc$ = MID$(x$, 1, 1)         'Top Left Corner
trc$ = MID$(x$, 2, 1)         'Top Right Corner
brc$ = MID$(x$, 3, 1)         'Bottom Right Corner
blc$ = MID$(x$, 4, 1)         'Bottom Left Corner
ver$ = MID$(x$, 5, 1)         'Vertical line
hor$ = MID$(x$, 6, 1)         'Horizontal line
tx$ = MID$(x$, 7, 1)          'Top intersection
rx$ = MID$(x$, 8, 1)          'Right intersection
bx$ = MID$(x$, 9, 1)          'Bottom intersection
lx$ = MID$(x$, 10, 1)         'Left intersection
mx$ = MID$(x$, 11, 1)         'Middle intersection

'                           *** Draw the Box ***

wide = col2 - col1 - 1

LOCATE row1, col1                                      'The main box outline
PRINT tlc$; STRING$(wide, hor$); trc$;                 '(just like SUB Box())
FOR x = (row1 + 1) TO (row2 - 1)
     LOCATE x, col1
     PRINT ver$;
     LOCATE x, col2
     PRINT ver$;
NEXT x
LOCATE row2, col1
PRINT blc$; STRING$(wide, hor$); brc$;

IF row3 > 0 THEN                                       'Split horizontally?
     LOCATE row3, col1
     PRINT lx$; STRING$(wide, hor$); rx$;
END IF

IF col3 > 0 THEN                                       'Split vertically?
     LOCATE row1, col3
     PRINT tx$;
     FOR x = (row1 + 1) TO (row2 - 1)
          LOCATE x, col3
          PRINT ver$;
     NEXT x
     LOCATE row2, col3
     PRINT bx$;
END IF

IF row3 > 0 AND col3 > 0 THEN                          'Split both ways?
     LOCATE row3, col3
     PRINT mx$
END IF

END SUB

FUNCTION Replace$ (orig$, old$, new$)
'****************************************************************************
'Replace all occurrences of a substring within a string with another.
'Example:  Replace$("Peter Piper","er","ey") --> "Petey Pipey"
'Supporting functions: none.
'****************************************************************************

IF orig$ = "" OR old$ = "" THEN         'If either argument is a null string,
     Replace$ = orig$                   'return the original because nothing
     EXIT FUNCTION                      'can be replaced in a null string,
END IF                                  'and INSTR always returns 1 when
                                        'searching for a null string.
 
rep$ = orig$                            'Don't alter the original in case the
                                        'calling program still needs it!

x = INSTR(rep$, old$)                   'Find the first occurrence.

DO WHILE x > 0                          'Loop until old$ is no longer found.
     l$ = LEFT$(rep$, x - 1)            'Get the part to the left of old$.
     r$ = MID$(rep$, x + LEN(old$))     'Get the part to the right of old$.
     rep$ = l$ + new$ + r$              'Replace old$ with new$.
     start = LEN(l$) + LEN(new$) + 1    'Start the next search after the
     x = INSTR(start, rep$, old$)       'part we just replaced (in case old$
LOOP                                    'is a substring of new$).

Replace$ = rep$                         'Return the altered string

END FUNCTION

FUNCTION Rinstr (start, search$, lookfor$)
'****************************************************************************
'Kind of a "Reverse INSTR" (hence the name).  Returns the character position
'of the LAST occurrence of a substring within another.  If the start argument
'is greater than zero, search$ is truncated at (start-1) before the search
'begins (I would rather have called it "end" but that word is taken).  The
'start argument is especially useful for subsequent calls to Rinstr, i.e. to
'find the second-to-last occurrence, etc.
'
'Examples:  Rinstr(0,"Peter Piper","er")  --> 10
'           Rinstr(10,"Peter Piper","er") --> 4  (Searches "Peter Pip")
'
'Supporting functions: none.
'****************************************************************************

IF search$ = "" OR lookfor$ = "" THEN   'If either argument is a null string,
     Rinstr = 0                         'return zero.  (I hate that INSTR
     EXIT FUNCTION                      'returns a 1 when you search for a
END IF                                  'null string!)

IF start > LEN(search$) OR start < 0 THEN    'Return zero if start is greater
     Rinstr = 0                              'than the length of search$ or
     EXIT FUNCTION                           'is negative.
END IF

IF start > 0 THEN                       'If start > 0, chop the string at
     s$ = LEFT$(search$, start - 1)     'that position so we don't bother
ELSE                                    'searching past it.
     s$ = search$                       'Otherwise, just use the original.
END IF

last = 0
x = INSTR(s$, lookfor$)                      'Get the first occurrence.

WHILE x > 0                                  'Go through the string,
     last = x                                'increasing the starting
     x = INSTR((last + 1), s$, lookfor$)     'position each time the
WEND                                         'substring is found.

Rinstr = last            'Return the position of the rightmost occurrence.

END FUNCTION

SUB Rise (text$, row, col)
'****************************************************************************
'Prints text vertically on the screen, rising from the specified row and
'column position.  If the length of the text would continue above row 1,
'printing will stop at that point.  See also: SUB Drop()
'
'Supporting functions: none.
'****************************************************************************

r = row                                 'We will need to alter the row value.
FOR x = 1 TO LEN(text$)
     LOCATE r, col
     PRINT MID$(text$, x, 1);
     r = r - 1
     IF r = 0 THEN EXIT FOR             'Quit if we reach the top of the
NEXT x                                  'screen.

END SUB

FUNCTION Scramble$ (text$)
'****************************************************************************
'This function randomly scrambles a line of text.  I can't think of any real
'use for it, but it is a good example of how to shuffle an array.  You may
'find it handy should you want to write a card game (or word puzzles!).
'
'Supporting functions: none.
'****************************************************************************

IF text$ = "" THEN                 'Don't bother with a null string.
     Scramble$ = ""
     EXIT FUNCTION
END IF

l = LEN(text$)                     'Find out the length of text$ and create
DIM ltr$(1 TO l)                   'an array to hold each letter.

FOR x = 1 TO l                     'Fill the array with the letters.
     ltr$(x) = MID$(text$, x, 1)
NEXT x

FOR x = 1 TO l                     'Go through the array, swapping each
     e = INT(RND * l) + 1          'element with another random element.
     SWAP ltr$(x), ltr$(e)         'This makes sure each element has been
NEXT x                             'swapped at least once, probably more.

FOR x = 1 TO l                     'Put the letters back together.
     new$ = new$ + ltr$(x)
NEXT x

ERASE ltr$                         'Release the array.

Scramble$ = new$                   'Return the scrambled string.

END FUNCTION

SUB SlideL (text$, row, col, delay)
'****************************************************************************
'Just try it and see.  See SUB TeleType for an explanation of the delay
'argument.
'Supporting functions: none.
'****************************************************************************

d! = delay                              'Convert delay from an integer to a
                                        'single precision variable.

IF d! < 1 THEN d! = 5                   'Assign the default delay of 1/20th
                                        'of a second (5/100).

d! = d! / 100                           'Change delay to 100ths of a second.

FOR x = 1 TO LEN(text$)
   
     IF (col - x + 1) < 1 THEN          'If the column position becomes
          EXIT FOR                      'illegal (0 or less), stop printing.
     END IF

     LOCATE row, col - x + 1            'Print one character at a time,
     PRINT LEFT$(text$, x);             'from left to right.
   
     now! = TIMER                       'Get current value of TIMER.
     WHILE TIMER < (now! + d!): WEND    'Wait for timer to increase by d!
   
     IF INKEY$ <> "" THEN d! = 0        'If a key is pressed, stop delaying.

NEXT x

END SUB

SUB SlideR (text$, row, col, delay)
'****************************************************************************
'Just try it and see.  See SUB TeleType for an explanation of the delay
'argument.
'Supporting functions: none.
'****************************************************************************

d! = delay                              'Convert delay from an integer to a
                                        'single precision variable.

IF d! < 1 THEN d! = 5                   'Assign the default delay of 1/20th
                                        'of a second (5/100).

d! = d! / 100                           'Change delay to 100ths of a second.

FOR x = 1 TO LEN(text$)
    
     LOCATE row, col                    'Print one character at a time,
     PRINT RIGHT$(text$, x);            'from right to left.
    
     now! = TIMER                       'Get current value of TIMER.
     WHILE TIMER < (now! + d!): WEND    'Wait for timer to increase by d!
    
     IF INKEY$ <> "" THEN d! = 0        'If a key is pressed, stop delaying.

NEXT x

END SUB

SUB Spread (text$, row, col, delay)
'****************************************************************************
'Just try it and see.  See SUB TeleType for an explanation of the delay
'argument.
'Supporting functions: none.
'****************************************************************************

d! = delay                              'Convert delay from an integer to a
                                        'single precision variable.

IF d! < 1 THEN d! = 5                   'Assign the default delay of 1/20th
                                        'of a second (5/100).

d! = d! / 100                           'Change delay to 100ths of a second.

IF text$ = "" THEN EXIT SUB             'Quit if text$ is a null string.

txt$ = text$                            'Use a temp variable to modify text$.

IF LEN(txt$) MOD 2 = 1 THEN             'If the length of text$ is not even,
     txt$ = txt$ + " "                  'add a space to make it even.
END IF

lt$ = LEFT$(txt$, LEN(txt$) \ 2)        'Break it into left & right sides.
rt$ = RIGHT$(txt$, LEN(txt$) \ 2)

FOR x = 1 TO LEN(rt$)                   'Could also use lt$, they're equal.
  
     LOCATE row, col                    'Print a letter from the right side.
     PRINT RIGHT$(rt$, x);
    
     IF (col - x) >= 1 THEN             'Only print the left side while the
                                        'value of column is still valid.
          LOCATE row, col - x
          PRINT LEFT$(lt$, x);          'Print a letter from the left side.
     END IF
  
     now! = TIMER                       'Get current value of TIMER.
     WHILE TIMER < (now! + d!): WEND    'Wait for timer to increase by d!
  
     IF INKEY$ <> "" THEN d! = 0        'If a key is pressed, stop delaying.

NEXT x

END SUB

FUNCTION Squeeze$ (orig$, char$)
'****************************************************************************
'Removes all occurrences of a substring from within a string.
'Example: Squeeze$("Peter Piper","er") --> "Pet Pip"
'Supporting functions: none.
'****************************************************************************

IF orig$ = "" OR char$ = "" THEN        'If either argument is a null string,
     Squeeze$ = orig$                   'return the original because nothing
     EXIT FUNCTION                      'can be squeezed from a null string
END IF                                  'and INSTR always returns 1 when
                                        'searching for a null string.
  
new$ = orig$                            'Don't alter the original in case the
                                        'calling program still needs it!

DO WHILE INSTR(new$, char$) > 0         'Loop until char$ is no longer found.
    
     x = INSTR(new$, char$)             'Find the location of char$.
     l$ = LEFT$(new$, x - 1)            'Get the part to the left of char$.
     r$ = MID$(new$, x + LEN(char$))    'Get the part to the right of char$.
     new$ = l$ + r$                     'Put the string back together without
                                        'char$!
LOOP

Squeeze$ = new$                         'Return the squeezed string

END FUNCTION

FUNCTION Strip$ (orig$, side$, char$)
'****************************************************************************
'Strips leading and/or trailing characters from a string.  The argument side$
'may be specified as "L" to strip the left side, "R" to strip the right side,
'or any other value to strip both sides.  This is a souped-up version of the
'LTRIM$ and RTRIM$ functions.
'
'Example:  Strip$("00100","L","0") --> "100"
'          Strip$("AABAA","R","A") --> "AAB"
'          Strip$("00100","","0")  --> "1"
'          Strip$("     ","L"," ") --> ""
'      
'Supporting functions: none.
'****************************************************************************

IF orig$ = "" OR char$ = "" THEN        'If either argument is a null string,
     Strip$ = orig$                     'return the original because nothing
     EXIT FUNCTION                      'can be stripped from a null string,
END IF                                  'and you can't search for a null.

new$ = orig$                            'Don't alter the originals in case
s$ = UCASE$(LEFT$(side$, 1))            'the calling program still needs 'em!

IF s$ <> "R" THEN                       'Strip the left side as long as
     DO WHILE LEFT$(new$, 1) = char$    'anything except "R" was asked for.
          new$ = MID$(new$, 2)
     LOOP
END IF

IF s$ <> "L" THEN                       'Do the same for the right side.
     DO WHILE RIGHT$(new$, 1) = char$
          new$ = LEFT$(new$, LEN(new$) - 1)
     LOOP
END IF

Strip$ = new$                           'Return the stripped string.

END FUNCTION

FUNCTION Stuff$ (orig$, char$, position)
'****************************************************************************
'Inserts character(s) into a string at the specified character position.
'Examples: Stuff$("QBasic","uick",2)   --> "QuickBasic"
'          Stuff$("Landlocked","-",5)  --> "Land-locked"
'Supporting functions: none.
'****************************************************************************

part1$ = LEFT$(orig$, position - 1)     'Break the string into two parts at
part2$ = MID$(orig$, position)          'the specified position.

Stuff$ = part1$ + char$ + part2$        'Rebuild the string, including char$
                                        'between the two parts.
END FUNCTION

SUB TeleType (text$, delay)
'****************************************************************************
'Prints text one character at a time beginning at the current cursor location.
'The delay between each character being printed is measured in 1/100ths of a
'second (a delay of 100 would equal one second).  If a value of zero or less
'is specified, the delay defaults to 5/100ths of a second.  If a key is
'pressed during the SUB, the remainder of the string is printed without any
'delay.
'You could easily add some sound to this procedure.  I recommend using SOUND
'20000,1 after each letter except spaces and a delay of at least 7.
'
'Supporting functions: none.
'****************************************************************************

d! = delay                              'Convert delay from an integer to a
                                        'single precision variable.

IF d! < 1 THEN d! = 5                   'Assign the default delay of 1/20th
                                        'of a second (5/100).

d! = d! / 100                           'Change delay to 100ths of a second.

FOR x = 1 TO LEN(text$)                 'Print text$ one character at a time.
    
     PRINT MID$(text$, x, 1);
    
     now! = TIMER                       'Get current value of TIMER.
     WHILE TIMER < (now! + d!): WEND    'Wait for timer to increase by d!.
    
     IF INKEY$ <> "" THEN d! = 0        'If a key is pressed, stop delaying.

NEXT x

END SUB

SUB Wipe (row)
'****************************************************************************
'Clears a row on the screen.
'Supporting functions: none.
'****************************************************************************

LOCATE row, 1            'Fill the row with spaces & place the cursor back at
PRINT SPACE$(80);        'the beginning of the row.  Very simple, but handy.
LOCATE row, 1

END SUB

SUB WipeArea (row1, col1, row2, col2)
'****************************************************************************
'Clears an area of the screen.
'Supporting functions: none.
'****************************************************************************

wide = col2 - col1 + 1
FOR x = row1 TO row2               'Fill an area of the screen with spaces
     LOCATE x, col1                'and relocate the cursor back at the top
     PRINT SPACE$(wide);           'left corner of the area.
NEXT x
LOCATE row1, col1

END SUB

