'
'  -- MODEM_IO.BAS --
'
' This program is donated to the Public
' Domain by MarshallSoft Computing, Inc.
' It is provided as an example of the use
' of the Personal Communications Library.
'
$CPU 8086          'make compatible with XT systems
$LIB ALL OFF       'turn off all PowerBASIC libraries
$ERROR ALL ON      'turn on all PowerBASIC error checking
$OPTIMIZE SIZE     'optimize for smaller code
$COMPILE UNIT      'compile to a UNIT (.PBU)

DEFINT A-Z         'Required for all numeric functions, forces PB to not
                   'include floating point routines in UNIT (makes it smaller)

$INCLUDE "PCL4PB.BI"

%FALSE = 0
%TRUE  = NOT %FALSE


FUNCTION BreakTest() PUBLIC

  ' User BREAK ?
  IF (INKEY$ <> "" ) OR SioBrkKey THEN
    PRINT "User BREAK"
    BreakTest = %TRUE
    EXIT FUNCTION
  END IF

  BreakTest = %FALSE

END FUNCTION


' enter command state
' NOTE: assumes escape char = '+' & guard time = 1 sec

FUNCTION ModemCmdState(BYVAL Port AS INTEGER) PUBLIC

  DIM Code AS INTEGER
  DIM I    AS INTEGER

  ' delay a bit over 1 second
  Code = SioDelay(25)

  ' send Escape Code exactly 3 times
  FOR I = 1 TO 3
    Code = SioPutc(Port, ASC("+"))
    Code = SioDelay(5)
  NEXT I

  ' delay again
  Code = SioDelay(25)

END FUNCTION


' echos incoming to screen

FUNCTION ModemEcho(BYVAL Port AS INTEGER, _
                   BYVAL Echo AS INTEGER) PUBLIC
  DIM Time AS LONG
  DIM Code AS INTEGER

  Time = SioTimer
  DO WHILE SioTimer < Time + Echo
    Code = SioGetc(Port, 1)
    IF Code = 13 THEN
      PRINT
    ELSEIF Code >= &H20 THEN
      PRINT CHR$(Code);
    END IF
  LOOP

END FUNCTION


' wait for continuous quiet (no incoming serial data)

FUNCTION ModemQuiet(BYVAL Port AS INTEGER, _
                    BYVAL Tics AS INTEGER) PUBLIC
  DIM CharTime AS LONG
  DIM Code     AS INTEGER

  ' set up
  CharTime = SioTimer

  DO
    ' User BREAK ?
    IF BreakTest THEN
      ModemQuiet = %FALSE
      EXIT FUNCTION
    END IF

    ' wait for next character
    Code = SioGetc(Port, 1)
    IF Code < -1 THEN
      ModemQuiet = %FALSE
      EXIT FUNCTION
    END IF
    IF Code >= 0 THEN
      CharTime = SioTimer
      IF Code = 10 THEN
        PRINT
      ELSEIF Code >= &H20 THEN
        PRINT CHR$(Code);
      END IF
    ELSE
      ' =-1, timed out
      IF SioTimer >= CharTime + Tics THEN
         ModemQuiet = %TRUE
         EXIT FUNCTION
      END IF
    END IF
  LOOP

END FUNCTION


' send string to modem & get echo

FUNCTION ModemSendTo(BYVAL Port AS INTEGER, _
                     BYVAL Pace AS INTEGER, _
                     Text AS STRING) PUBLIC
  DIM I       AS INTEGER
  DIM Code    AS INTEGER
  DIM TheByte AS INTEGER
  DIM TheChar AS STRING

  FOR I = 1 TO LEN(Text$)
    ' User BREAK ?
    IF BreakTest THEN
      ModemSendTo = %FALSE
      EXIT FUNCTION
    END IF
    ' delay <Pace> tics
    IF Pace > 0 THEN Code = ModemEcho(Port, Pace)
    ' fetch character
    TheChar$ = MID$(Text$, I, 1)
    TheByte = ASC(TheChar$)

    SELECT CASE TheChar$
      CASE "!"
        ' replace ! with carriage return
        TheByte = 13
      CASE "~"
        ' delay 1/2 second
        Code = SioDelay(10)
        TheByte = &H20
      CASE " "
        ' delay 1/4 second
        Code = SioDelay(5)
        TheByte = &H20
    END SELECT

    ' transmit
    Code = SioPutc(Port, TheByte)
  NEXT I

  ModemSendTo = %TRUE

END FUNCTION

' wait for Text$

FUNCTION ModemWaitFor(BYVAL Port     AS INTEGER, _
                      BYVAL Tics     AS INTEGER, _
                      BYVAL CaseFlag AS INTEGER, _
                            Text     AS STRING) PUBLIC
  DIM Time   AS LONG
  DIM C1     AS STRING
  DIM C2     AS STRING
  DIM Code   AS INTEGER
  DIM First  AS INTEGER
  DIM Length AS INTEGER
  DIM I      AS INTEGER

  Length = LEN(Text$)

  ' wait for string
  Time = SioTimer
  First = 1
  DO WHILE SioTimer < Time + Tics
    ' User BREAK ?
    IF BreakTest THEN
      ModemWaitFor = %FALSE
      EXIT FUNCTION
    END IF
    ' wait for next character
    Code = SioGetc(Port, 1)
    IF Code < -1 THEN
      ModemWaitFor = %FALSE
      EXIT FUNCTION
    END IF
    IF Code >= 0 THEN
      ' echo char
      IF Code = 10 THEN
        PRINT
      ELSEIF Code >= &H20 THEN
        PRINT CHR$(Code);
      END IF
      ' prepare chars
      C1$ = MID$(Text$, First, 1)
      First = First + 1
      C2$ = CHR$(Code)
      ' case sensitive ?
      IF NOT CaseFlag THEN
        C1$ = UCASE$(C1$)
        C2$ = UCASE$(C2$)
      END IF
      ' does char match ?
      IF C1$ = C2$ THEN
        IF First > Length THEN
          ModemWaitFor = %TRUE
          EXIT FUNCTION
        END IF
      ELSE
        'start over again
        First = 1
      END IF
    END IF
  LOOP

  ModemWaitFor = %FALSE

END FUNCTION

' hangup phone (in command state)

FUNCTION ModemHangup(BYVAL Port AS INTEGER) PUBLIC

  DIM Code AS INTEGER

  ' enter command state
  Code = ModemCmdState(Port)

  ' hangup !
  Code = ModemSendTo(Port, 4, "!AT!")
  Code = ModemEcho(Port, 10)
  Code = ModemSendTo(Port, 4, "ATH0!")

END FUNCTION
